@@ -43,8 +43,8 @@ private[ui] class ApplicationPage(parent: MasterWebUI) extends WebUIPage("app")
       return UIUtils.basicSparkPage(request, msg, "Not Found")
     }
 
-    val executorHeaders = Seq("ExecutorID", "Worker", "Cores", "Memory", "Resources",
-      "State", "Logs")
+    val executorHeaders = Seq("ExecutorID", "Worker", "Cores", "Memory", "Resource Profile Id",
+      "Resources", "State", "Logs")
     val allExecutors = (app.executors.values ++ app.removedExecutors).toSet.toSeq
     // This includes executors that are either still running or have exited cleanly
     val executors = allExecutors.filter { exec =>
@@ -76,17 +76,17 @@ private[ui] class ApplicationPage(parent: MasterWebUI) extends WebUIPage("app")
                     data-placement="top">
                 <strong>Executor Limit: </strong>
                 {
-                  if (app.executorLimit == Int.MaxValue) "Unlimited" else app.executorLimit
+                  if (app.getExecutorLimit == Int.MaxValue) "Unlimited" else app.getExecutorLimit
                 }
                 ({app.executors.size} granted)
               </span>
             </li>
             <li>
-              <strong>Executor Memory:</strong>
+              <strong>Executor Memory - Default Resource Profile:</strong>
               {Utils.megabytesToString(app.desc.memoryPerExecutorMB)}
             </li>
             <li>
-              <strong>Executor Resources:</strong>
+              <strong>Executor Resources - Default Resource Profile:</strong>
               {formatResourceRequirements(app.desc.resourceReqsPerExecutor)}
             </li>
             <li><strong>Submit Date:</strong> {UIUtils.formatDate(app.submitDate)}</li>
@@ -145,6 +145,7 @@ private[ui] class ApplicationPage(parent: MasterWebUI) extends WebUIPage("app")
       </td>
       <td>{executor.cores}</td>
       <td>{executor.memory}</td>
+      <td>{executor.rpId}</td>
       <td>{formatResourcesAddresses(executor.resources)}</td>
       <td>{executor.state}</td>
       <td>
