@@ -47,6 +47,7 @@ class InMemoryCatalogedDDLSuite extends DDLSuite with SharedSparkSession {
     try {
       // drop all databases, tables and functions after each test
       spark.sessionState.catalog.reset()
+      spark.sessionState.catalogManager.reset()
     } finally {
       Utils.deleteRecursively(new File(spark.sessionState.conf.warehousePath))
       super.afterEach()
@@ -1120,7 +1121,7 @@ abstract class DDLSuite extends QueryTest with DDLSuiteBase {
     sql("ALTER TABLE dbx.tab1 SET TBLPROPERTIES ('andrew' = 'or14', 'kor' = 'bel')")
     assert(getProps == Map("andrew" -> "or14", "kor" -> "bel"))
     // set table properties without explicitly specifying database
-    catalog.setCurrentDatabase("dbx")
+    spark.sessionState.catalogManager.setCurrentNamespace(Array("dbx"))
     sql("ALTER TABLE tab1 SET TBLPROPERTIES ('kor' = 'belle', 'kar' = 'bol')")
     assert(getProps == Map("andrew" -> "or14", "kor" -> "belle", "kar" -> "bol"))
     // table to alter does not exist
@@ -1154,7 +1155,7 @@ abstract class DDLSuite extends QueryTest with DDLSuiteBase {
     sql("ALTER TABLE dbx.tab1 UNSET TBLPROPERTIES ('j')")
     assert(getProps == Map("p" -> "an", "c" -> "lan", "x" -> "y"))
     // unset table properties without explicitly specifying database
-    catalog.setCurrentDatabase("dbx")
+    spark.sessionState.catalogManager.setCurrentNamespace(Array("dbx"))
     sql("ALTER TABLE tab1 UNSET TBLPROPERTIES ('p')")
     assert(getProps == Map("c" -> "lan", "x" -> "y"))
     // table to alter does not exist
