@@ -189,7 +189,9 @@ class HiveClientSuite(version: String, allVersions: Seq[String])
       client.dropDatabase("temporary", ignoreIfNotExists = false, cascade = false)
       assert(false, "dropDatabase should throw HiveException")
     }
-    assert(ex.message.contains("Cannot drop a non-empty database: temporary."))
+    checkError(ex,
+      errorClass = "SCHEMA_NOT_EMPTY",
+      parameters = Map("schemaName" -> "`temporary`"))
 
     client.dropDatabase("temporary", ignoreIfNotExists = false, cascade = true)
     assert(!client.databaseExists("temporary"))
@@ -413,7 +415,7 @@ class HiveClientSuite(version: String, allVersions: Seq[String])
 
   test("getPartitionsByFilter") {
     // Only one partition [1, 1] for key2 == 1
-    val result = client.getPartitionsByFilter(client.getTable("default", "src_part"),
+    val result = client.getPartitionsByFilter(client.getRawHiveTable("default", "src_part"),
       Seq(EqualTo(AttributeReference("key2", IntegerType)(), Literal(1))))
 
     // Hive 0.12 doesn't support getPartitionsByFilter, it ignores the filter condition.
@@ -437,7 +439,7 @@ class HiveClientSuite(version: String, allVersions: Seq[String])
 
   test("getPartitionOption(table: CatalogTable, spec: TablePartitionSpec)") {
     val partition = client.getPartitionOption(
-      client.getTable("default", "src_part"), Map("key1" -> "1", "key2" -> "2"))
+      client.getRawHiveTable("default", "src_part"), Map("key1" -> "1", "key2" -> "2"))
     assert(partition.isDefined)
   }
 
@@ -525,10 +527,13 @@ class HiveClientSuite(version: String, allVersions: Seq[String])
       storageFormat))
     try {
       client.createPartitions("default", "src_part", partitions, ignoreIfExists = false)
-      val errMsg = intercept[PartitionsAlreadyExistException] {
+      val e = intercept[PartitionsAlreadyExistException] {
         client.createPartitions("default", "src_part", partitions, ignoreIfExists = false)
-      }.getMessage
-      assert(errMsg.contains("partitions already exists"))
+      }
+      checkError(e,
+        errorClass = "PARTITIONS_ALREADY_EXIST",
+        parameters = Map("partitionList" -> "PARTITION (`key1` = 101, `key2` = 102)",
+          "tableName" -> "`default`.`src_part`"))
     } finally {
       client.dropPartitions(
         "default",
@@ -666,8 +671,16 @@ class HiveClientSuite(version: String, allVersions: Seq[String])
       client.runSqlHive("CREATE TABLE materialized_view_tbl (c1 INT)")
       client.runSqlHive(
         s"CREATE MATERIALIZED VIEW mv1 $disableRewrite AS SELECT * FROM materialized_view_tbl")
-      val e = intercept[AnalysisException](versionSpark.table("mv1").collect()).getMessage
-      assert(e.contains("Hive materialized view is not supported"))
+      checkError(
+        exception = intercept[AnalysisException] {
+          versionSpark.table("mv1").collect()
+        },
+        errorClass = "UNSUPPORTED_FEATURE.HIVE_TABLE_TYPE",
+        parameters = Map(
+          "tableName" -> "`mv1`",
+          "tableType" -> "materialized view"
+        )
+      )
     }
   }
 
@@ -895,7 +908,7 @@ class HiveClientSuite(version: String, allVersions: Seq[String])
   test("Decimal support of Avro Hive serde") {
     val tableName = "tab1"
     // TODO: add the other logical types. For details, see the link:
-    // https://avro.apache.org/docs/1.11.0/spec.html#Logical+Types
+    // https://avro.apache.org/docs/1.11.2/specification/#logical-types
     val avroSchema =
     """{
       |  "name": "test_record",
@@ -936,8 +949,15 @@ class HiveClientSuite(version: String, allVersions: Seq[String])
         if (isPartitioned) {
           val insertStmt = s"INSERT OVERWRITE TABLE $tableName partition (ds='a') SELECT 1.3"
           if (version == "0.12" || version == "0.13") {
-            val e = intercept[AnalysisException](versionSpark.sql(insertStmt)).getMessage
-            assert(e.contains(errorMsg))
+            checkError(
+              exception = intercept[AnalysisException](versionSpark.sql(insertStmt)),
+              errorClass = "INCOMPATIBLE_DATA_FOR_TABLE.CANNOT_SAFELY_CAST",
+              parameters = Map(
+                "tableName" -> "`spark_catalog`.`default`.`tab1`",
+                "colName" -> "`f0`",
+                "srcType" -> "\"DECIMAL(2,1)\"",
+                "targetType" -> "\"BINARY\"")
+            )
           } else {
             versionSpark.sql(insertStmt)
             assert(versionSpark.table(tableName).collect() ===
@@ -946,8 +966,15 @@ class HiveClientSuite(version: String, allVersions: Seq[String])
         } else {
           val insertStmt = s"INSERT OVERWRITE TABLE $tableName SELECT 1.3"
           if (version == "0.12" || version == "0.13") {
-            val e = intercept[AnalysisException](versionSpark.sql(insertStmt)).getMessage
-            assert(e.contains(errorMsg))
+            checkError(
+              exception = intercept[AnalysisException](versionSpark.sql(insertStmt)),
+              errorClass = "INCOMPATIBLE_DATA_FOR_TABLE.CANNOT_SAFELY_CAST",
+              parameters = Map(
+                "tableName" -> "`spark_catalog`.`default`.`tab1`",
+                "colName" -> "`f0`",
+                "srcType" -> "\"DECIMAL(2,1)\"",
+                "targetType" -> "\"BINARY\"")
+            )
           } else {
             versionSpark.sql(insertStmt)
             assert(versionSpark.table(tableName).collect() ===
